<?php

namespace Teknisa\MAN\Controllers;


use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Services;

class CadConsumidorDados extends CrudWithParameterBag {
    protected $dataSourceName = 'cadConsumidor_dados';
    protected $parameterBagColumns = array('CDCLIENTE','CDCONSUMIDOR','NRORG');
    protected $utilService;
    protected $ConsumidorService;
    protected $ConsumidorConsService;
    protected $salesMessage;
    protected $criptoPasswordUrl;
    protected $xmlSchemaUrl;
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag,
        $ConsumidorService, Utilities\Util $utilService, Services\CadConsumidorDados $ConsumidorConsService, Utilities\SalesMessage $salesMessage, $criptoPasswordUrl, $xmlSchemaUrl) {
        $this->utilService           = $utilService;
        $this->ConsumidorService     = $ConsumidorService;
        $this->consumidorConsService = $ConsumidorConsService;
        $this->salesMessage          = $salesMessage;
        $this->criptoPasswordUrl     = $criptoPasswordUrl;
        $this->xmlSchemaUrl          = $xmlSchemaUrl;
        parent::__construct($dataSourceManager, $parameterBag);
        
    }
    
    
     public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            if(!isset($dataSet->getRows()[0]['CDVENDEDOR'])) {
                $dataSet->getRows()[0]['CDVENDEDOR'] = '';
            }
            
            $checkCodigo = $this->ConsumidorService->checkConsumidores(
                $dataSet->getRows()[0]['CDCONSUMIDOR'],
                $dataSet->getRows()[0]['NRORG'],
                $dataSet->getRows()[0]['CDCLIENTE']
            );
            
            if (count($checkCodigo) > 0 && 
                $dataSet->getRows()[0]['__is_new'] == true) {
                    
                if($checkCodigo[0]['IDATIVO'] == 'N'){
                    $response->addMessage(new Message("Este código de consumidor já está registrado, porém, inativo.",Message::TYPE_WARNING)); 
                    $response->addDataSet(new DataSet($this->dataSourceName, array(false)));
                    $response->setError(new DTO\Response\Error("Este código de consumidor já está registrado, porém, inativo.", '02'));
                } else {
                    $response->addMessage(new Message("Já existe um consumidor com esse Código!", Message::TYPE_WARNING)); 
                    $response->addDataSet(new DataSet($this->dataSourceName, array(false)));
                    $response->setError(new DTO\Response\Error("Já existe um consumidor com esse Código!", '01'));
                }
            } else {
                if(!empty($dataSet->getRows()[0]['CDSENHACONS'])) {
                    $senhaCriptografada =  $this->getDelphiPasswordEncrypted($dataSet->getRows()[0]['CDSENHACONS']);
                    $dataSet->getRows()[0]['CDSENHACONS'] = $senhaCriptografada;
                    $dataSet->getRows()[0]['NRCEPCONS'] = preg_replace("/[^0-9]/", "", $dataSet->getRows()[0]['NRCEPCONS']);
                    $dataSet->getRows()[0]['NRTELECONS'] = preg_replace("/[^0-9]/", "", $dataSet->getRows()[0]['NRTELECONS']);
                    $dataSet->getRows()[0]['NRCELULARCONS'] = preg_replace("/[^0-9]/", "", $dataSet->getRows()[0]['NRCELULARCONS']);
                }
                $persistedRows = $this->dataSourceManager->persist($dataSet);
                $response->addDataSet(new DataSet($this->dataSourceName, array(true)));
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    
    public function findConsResp(DTO\Request $request, DTO\Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            
            $result = $this->consumidorConsService->findConsResp($params);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('cadConsumidorResp_dados', $result));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function saveConsResp(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows();
            $validCodigo = $this->consumidorConsService->validCodigo($row);
            
            if (empty($validCodigo)){
                $this->consumidorConsService->saveConsResp($row);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('cadConsumidorResp_dados'));
            } else {
                throw new \Exception ("O Código Mifare já está cadastrado para outro consumidor do mesmo cliente.");
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function changePasswordConsResp(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows();
            
            $this->consumidorConsService->changePasswordConsResp($row);
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('changePasswordConsResp'));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
   
   public function getDelphiPasswordEncrypted($password) {
        // Parameters to retrieve the crypted password from a server 
        $options = array(
            'encoding' => 'ISO-8859-1',
            'trace' => true,
            'exceptions' => true,
            'compression' => SOAP_COMPRESSION_ACCEPT | SOAP_COMPRESSION_GZIP,
        );

        $args = array(
            new \SoapVar($password, XSD_STRING, 'string', $this->xmlSchemaUrl)
        );

        $params = array(
            'rdm' => 'Bib_html.rdmDBGeralHtm',
            'method' => 'GetCriptoPass',
            'args' => $args,
        );
        
        // Tries to get the encrypted password
        try {
            $client = new \SoapClient($this->criptoPasswordUrl, $options);
            $result = $client->executeCOMMethod($params);
            $encryption = $result->executeCOMMethodResult;
        } catch (\SoapFault $e) {
            throw new \Exception($e->getMessage());
        }
        
        return $encryption;
    }
}