<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;

class ConsolidacaoAutomatica extends \Teknisa\MAN\Controllers\TeknisaCrud {
    
    protected $dataSourceManager;
    protected $crudService;
    protected $consolidacaoAutService;
    protected $salesMessage;
    
    public function __construct($dataSourceManager, $crudService, Services\ConsolidacaoAutomatica $consolidacaoAutService, Utilities\SalesMessage $salesMessage) {
        $this->dataSourceManager      = $dataSourceManager;
        $this->crudService            = $crudService;
        $this->consolidacaoAutService = $consolidacaoAutService;
        $this->salesMessage           = $salesMessage;
    }
    
    public function buscaConsolidacoes(DTO\Request $request, DTO\Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $criterioestabelec = $this->consolidacaoAutService->buscaConsolidacoes($params);

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('vnd04600_ConsolidacaoAutomatica', $criterioestabelec));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function saveConsolidacoes(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows();
            $params[0]['value']  = $row[0]['CDFILIAL'];
            $params[1]['value']  = $row[0]['CDLOJA'];
            $params[2]['value']  = $row[0]['NRORG'];
            $result = $this->consolidacaoAutService->buscaConsolidacoes($params);
            if(count($result) >0){
                $row[0]["__is_new"] = false;
            } else {
                $row[0]["__is_new"] = true;
            }
            if($row[0]["__is_new"] == true){
                $this->consolidacaoAutService->insertConsolidacoes($row);
                $message= array($this->salesMessage->getMessage('criado'));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('vnd04600_ConsolidacaoAutomatica', $message));
            } else {
                $criterioestabelec = $this->consolidacaoAutService->updateConsolidacoes($row);
                $message= array($this->salesMessage->getMessage('alterado'));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('vnd04600_ConsolidacaoAutomatica', $message));
            }

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Metodo para alterar o horário para um formato 
     * que o Jenkins aceita.
     * @param $horario
     * @param $intervalo
     * @return $novoHorario
     */
    private function alteraHorario($horario, $intervalo) {
        $novoHorario = "* * * * *";
        $hora = substr($horario, 0, -2);
        $minuto = substr($horario, 2);
        
        if($intervalo === "D") {
            $novoHorario = $minuto . " " . $hora . " * * *";
        } else if($intervalo === "I") {
            $novoHorario = $minuto . " H/" . $hora . " * * *";
        }
        
        return $novoHorario;
    }
    
    /**
     * Metodo que cria o Job no Jenkins passando os parâmetros para o arquivo
     * que realizará a consolidação das vendas periodicamente.
     * @param $nomeJob
     * @param $descricao
     * @param $dataHora
     * @param $cdFilial
     * @param $cdLoja
     * @param $dtIniConsVenAut
     * @param $cdFiliProc
     * @param $cdOperador
     * @param $nrorg
     */
    private function criaJob($nomeJob, $descricao, $dataHora, $cdFilial, $cdLoja, $dtIniConsVenAut, $cdFiliProc, $cdOperador, $nrorg) {
        try {
            $dataInicialApuracao = explode(' ', $dtIniConsVenAut);
            
            $urlCliente = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]";
            // Exemplos:
            // $urlCliente = "https://retail.teknisa.com"; // SAAS
            // $urlCliente = "https://giraffas-retail.teknisa.com"; // HOSTING

            $metodo = "curl -X POST " . $urlCliente . "/management/modules/teknisa/man/src/Teknisa/MAN/Utilities/ExecConsVen.php -d '". $cdFilial . "' '" . $cdLoja . "' '" . $dataInicialApuracao[0] . "' '" . $cdFiliProc . "' '" . $cdOperador . "' '" . $nrorg;
            
            $jenkins = new Jobs($nomeJob);
			
            $jenkins->setType(Jobs::TYPE_JOB_MULT_CONF);
            //$jenkins->setUrlJenkins("http://consolida.teknisa.cloud"); // URL do Jenkins
            $jenkins->setUrlConfig(null);
            $jenkins->setPeriod($dataHora);
            $jenkins->setDescription($descricao);
            $jenkins->setMethod($metodo);
            $jenkins->createJob();
            $jenkins->configuration();
        } catch(\Exception $e) {
            echo $e->getMessage();
            die;
        }
    }
    
    public function removeConsolidacoes(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows();
            $this->consolidacaoAutService->removeConsolidacoes($row);
            
            $message= array($this->salesMessage->getMessage('excluido'));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('vnd04600_ConsolidacaoAutomatica', $message));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }

}