<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Services\ExportPriceChartService;
use Teknisa\MAN\Utilities;

class ExportPriceChart extends CrudWithParameterBag {

    protected $dataSourceNameProdutos = 'getProdByTabePrec';
    protected $parameterBagColumns = array('CDFILIAL', 'CDTABEPREC', 'NRORG');
    protected $exportPriceChartService;
    protected $dataSourceName = 'exportPriceChart';

    public function __construct (DataSource\Manager $dataSourceManager, DataSource\ParameterBag $parameterBag, ExportPriceChartService $exportPriceChartService, Utilities\Util $utilService) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this->exportPriceChartService = $exportPriceChartService;
        $this->utilService = $utilService;
    }

    public function find(Request\Filter $request, Response $response){
        $this->dataSourceName = $this->dataSourceNameProdutos;
        parent::find($request, $response);
    }
  
    public function exportPriceChart(Request\Row $request, Response $response){
        // realiza rotina de Exportação da Tabela de Preço
        $returnData = Array(
            'STATUS'   => '',
            'UNIDADES' => []
        );
        
        $dataExport  = $request->getRow();
        $delPrecoDia = $dataExport['DATA']['DELETE_PRECODIA'] == 'N';
        $keepPreco   = $dataExport['DATA']['KEEP_PRECO'] == 'S';
        $isFilipc    = $dataExport['TYPE'] == 'E_FILIPC';
        
        try {
            if($dataExport['TYPE'] == 'E_MULTFILI') {
                $conditions = array(
                    'CDFILIALDES' => $dataExport['DATA']['CDFILIALDES'],
                    'NRORG'       => $dataExport['DATA']['NRORG']
                );
                
                // verifica se as unidades possuem tabela de preco
                $retorno = $this->exportPriceChartService->verificaUnidadeComTabelaDePreco($conditions);
                
                if($retorno['STATUS'] == 'OK') {
                    $returnData['STATUS'] = $this->exportPriceChartService->exportPriceChart($dataExport);
                } else {
                    $returnData['STATUS'] = $retorno['STATUS'];
                    $returnData['UNIDADES'] = $retorno['UNIDADES'];
                }
            } else {
                $returnData['STATUS'] = $this->exportPriceChartService->exportPriceChart($dataExport);
            }
            
            self::responseFrontEnd($response, true, $returnData);
        } catch (Exception $e) {
            self::responseFrontEnd($response, false, $e);
        }
    }

    public function responseFrontEnd(&$response, $type, $data){
        if ($type)
            $response->addDataSet(new DataSource\DataSet($this->dataSourceName, $data));
        else
            $response->setError(new Response\Error($data->getMessage(), $data->getCode()));
    }
    
    public function findVig(Request\Filter $request, Response $response){
        $conditions = $this->utilService->getParamsAsArray($request);
        self::responseFrontEnd($response, true,  $this->exportPriceChartService->findVig($conditions));
    }
}